@@ -1,4 +1,4 @@
-// $Header: /cvsroot/libcwd/libcwd/bfd.cc,v 1.175 2005/06/29 04:10:24 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/bfd.cc,v 1.185 2005/12/19 02:24:39 libcw Exp $
 //
 // Copyright (C) 2000 - 2004, by
 // 
@@ -18,7 +18,6 @@
 
 #if CWDEBUG_LOCATION
 
-#include <unistd.h>
 #include <cstdarg>
 #include <inttypes.h>
 #include <sys/stat.h>
@@ -192,13 +191,13 @@ void bfd_close(bfd* abfd)
       // cwbfd::
       typedef PTR addr_ptr_t;
 
-#ifndef HIDE_FROM_DOXYGEN	// Bug in doxygen
+      //! \cond HIDE_FROM_DOXYGEN
       // cwbfd::
       bool symbol_key_greater::operator()(symbol_ct const& a, symbol_ct const& b) const
       {
 	return symbol_start_addr(a.symbol) >= reinterpret_cast<char const*>(symbol_start_addr(b.symbol)) + symbol_size(b.symbol);
       }
-#endif
+      //! \endcond HIDE_FROM_DOXYGEN
 
       // cwbfd::
       char bfile_ct::ST_list_instance[sizeof(object_files_ct)] __attribute__((__aligned__));
@@ -270,10 +269,11 @@ void bfd_close(bfd* abfd)
 
       // cwbfd::
       void* const unknown_l_addr = (void*)-1;
+      void* const executable_l_addr = (void*)-2;
 
-#ifndef HIDE_FROM_DOXYGEN	// Bug in doxygen
+      //! \cond HIDE_FROM_DOXYGEN
       // cwbfd::
-      bfile_ct::bfile_ct(char const* filename, void* base) : M_lbase(base), M_object_file(filename)
+      bfile_ct::bfile_ct(char const* filename, void* base) : M_lbase(base), M_start_last_symbol(0), M_object_file(filename)
       {
 #if CWDEBUG_DEBUGM
 	LIBCWD_TSD_DECLARATION;
@@ -285,9 +285,9 @@ void bfd_close(bfd* abfd)
       }
 
 #if LIBCWD_THREAD_SAFE && CWDEBUG_ALLOC && __GNUC__ == 3 && __GNUC_MINOR__ == 4
-      void bfile_ct::initialize(char const* filename, void* base LIBCWD_COMMA_ALLOC_OPT(bool is_libc), bool is_libstdcpp LIBCWD_COMMA_TSD_PARAM)
+      bool bfile_ct::initialize(char const* filename LIBCWD_COMMA_ALLOC_OPT(bool is_libc), bool is_libstdcpp LIBCWD_COMMA_TSD_PARAM)
 #else
-      void bfile_ct::initialize(char const* filename, void* base LIBCWD_COMMA_ALLOC_OPT(bool is_libc) LIBCWD_COMMA_TSD_PARAM)
+      bool bfile_ct::initialize(char const* filename LIBCWD_COMMA_ALLOC_OPT(bool is_libc) LIBCWD_COMMA_TSD_PARAM)
 #endif
       {
 #if CWDEBUG_DEBUGM
@@ -304,7 +304,7 @@ void bfd_close(bfd* abfd)
 	  DoutFatal(dc::bfd, "bfd_openr: " << bfd_errmsg(bfd_get_error()));
 	M_abfd->cacheable = bfd_tttrue;
 #else
-	M_abfd = bfd::openr(filename, base != 0);
+	M_abfd = bfd::openr(filename);
 	M_abfd->M_s_end_offset = 0;
 #endif
 	M_abfd->usrdata = (addr_ptr_t)this;
@@ -339,7 +339,7 @@ void bfd_close(bfd* abfd)
 	  bfd_close(M_abfd);
 	  M_abfd = NULL;
 	  M_number_of_symbols = 0;
-	  return;
+	  return false;
 	}
 
 	long storage_needed = bfd_get_symtab_upper_bound(M_abfd);
@@ -353,7 +353,7 @@ void bfd_close(bfd* abfd)
 	  bfd_close(M_abfd);
 	  M_abfd = NULL;
 	  M_number_of_symbols = 0;
-	  return;
+	  return false;
 	}
 #endif
 
@@ -409,12 +409,11 @@ void bfd_close(bfd* abfd)
 	  if (!s_end_offset && M_number_of_symbols > 0)
 	  {
 #if CWDEBUG_ALLOC
-	    int saved_internal = __libcwd_tsd.internal;
 	    __libcwd_tsd.internal = 0;
 #endif
 	    Dout(dc::warning, "Cannot find symbol _end");
 #if CWDEBUG_ALLOC
-	    __libcwd_tsd.internal = saved_internal;
+	    __libcwd_tsd.internal = 1;
 #endif
 	  }
 
@@ -424,12 +423,11 @@ void bfd_close(bfd* abfd)
 #ifdef HAVE_DLOPEN
 #if CWDEBUG_ALLOC
 	    LIBCWD_TSD_DECLARATION;
-	    int saved_internal = __libcwd_tsd.internal;
 	    __libcwd_tsd.internal = 0;
 #endif
 	    if (!real_dlopen.symptr)
 	      real_dlopen.symptr = dlsym(RTLD_NEXT, "dlopen");
-	    void* handle = real_dlopen.func(filename, RTLD_LAZY);
+	    void* handle = real_dlopen.func(NULL, RTLD_LAZY);		// filename is already loaded.
 	    if (!handle)
 	    {
 #if LIBCWD_THREAD_SAFE && CWDEBUG_DEBUG
@@ -437,22 +435,11 @@ void bfd_close(bfd* abfd)
 #endif
 	      char const* dlerror_str;
 	      dlerror_str = dlerror();
-	      DoutFatal(dc::fatal, "::dlopen(" << filename << ", RTLD_LAZY): " << dlerror_str);
+	      DoutFatal(dc::fatal, "::dlopen(NULL, RTLD_LAZY): " << dlerror_str);
 	    }
-	    char* val;
-	    if (s_end_offset && (val = (char*)dlsym(handle, "_end")))	// dlsym will fail when _end is a local symbol.
-	    {
-#if CWDEBUG_ALLOC
-	      __libcwd_tsd.internal = saved_internal;
-#endif
-	      M_lbase = val - s_end_offset;
-	    }
-	    else if (!statically_linked)
 #ifdef HAVE__DL_LOADED
+	    if (!statically_linked)
             {
-#if CWDEBUG_ALLOC
-	      __libcwd_tsd.internal = saved_internal;
-#endif
 	      for(link_map* p = *dl_loaded_ptr; p; p = p->l_next)
 	        if (!strcmp(p->l_name, filename))
 		{
@@ -461,10 +448,13 @@ void bfd_close(bfd* abfd)
 		}
 	    }
 #endif // HAVE__DL_LOADED
-	    if (M_lbase == 0 || M_lbase == unknown_l_addr)
+#if CWDEBUG_ALLOC
+	    __libcwd_tsd.internal = 1;
+#endif
+	    if (M_lbase == unknown_l_addr)
 	    {
 #if CWDEBUG_ALLOC
-	      __libcwd_tsd.internal = saved_internal;
+	      __libcwd_tsd.internal = 1;
 	      typedef std::map<void*, unsigned int, std::less<void*>,
 	                       _private_::internal_allocator::rebind<std::pair<void* const, unsigned int> >::other> start_values_map_ct;
 #else
@@ -488,20 +478,29 @@ void bfd_close(bfd* abfd)
 		  if (dlerror() == NULL)
 		  {
 #if CWDEBUG_ALLOC
-		    __libcwd_tsd.internal = saved_internal;
+		    __libcwd_tsd.internal = 1;
 #endif
-		    void* start = reinterpret_cast<char*>(val) - (*s)->value - sect->offset;
-		    std::pair<start_values_map_ct::iterator, bool> p = start_values.insert(std::pair<void* const, unsigned int>(start, 0));
-		    if (++(*(p.first)).second > best_count)
+		    void* start = reinterpret_cast<char*>(val) - (*s)->value - sect->vma;
+		    // Shared object seem to be loaded at boundaries of 4096 bytes (a page size).
+		    // It is possible that we looked up a symbol of the library we are looking
+		    // for and got a symbol in a different library with the same name. The chance
+		    // that that results in a load address that is a multiple is very small; make
+		    // use of that fact and skip random looking start addresses.
+		    if (((size_t)start & 0xfff) == 0)
 		    {
-		      best_start = start;
-		      if (++best_count == 10)	// Its unlikely that even more than 2 wrong values will have the same value.
-			break;			// So if we reach 10 then this is value we are looking for.
+		      std::pair<start_values_map_ct::iterator, bool> p =
+			  start_values.insert(std::pair<void* const, unsigned int>(start, 0));
+		      if (++(*(p.first)).second > best_count)
+		      {
+			best_start = start;
+			if (++best_count == 10)	// Its unlikely that even more than 2 wrong values will have the same value.
+			  break;			// So if we reach 10 then this is value we are looking for.
+		      }
 		    }
 		  }
 #if CWDEBUG_ALLOC
 		  else
-		    __libcwd_tsd.internal = saved_internal;
+		    __libcwd_tsd.internal = 1;
 #endif
 		}
 	      }
@@ -512,7 +511,7 @@ void bfd_close(bfd* abfd)
 #endif
 		Dout(dc::warning, "Unable to determine start of \"" << filename << "\", skipping.");
 #if CWDEBUG_ALLOC
-		__libcwd_tsd.internal = saved_internal;
+		__libcwd_tsd.internal = 1;
 #endif
 		free(M_symbol_table);
 		M_symbol_table  = NULL;
@@ -529,9 +528,9 @@ void bfd_close(bfd* abfd)
 		  real_dlclose.symptr = dlsym(RTLD_NEXT, "dlclose");
 		real_dlclose.func(handle);
 #if CWDEBUG_ALLOC
-		__libcwd_tsd.internal = saved_internal;
+		__libcwd_tsd.internal = 1;
 #endif
-		return;
+		return false;
 	      }
 	      M_lbase = best_start;
 	    }
@@ -543,25 +542,21 @@ void bfd_close(bfd* abfd)
 	    real_dlclose.func(handle);
 	    Dout(dc::continued, '(' << M_lbase << ") ... ");
 #if CWDEBUG_ALLOC
-	    __libcwd_tsd.internal = saved_internal;
+	    __libcwd_tsd.internal = 1;
 #endif
 #else // !HAVE_DLOPEN
 	    DoutFatal(dc::fatal, "Can't determine start of shared library: you will need libdl to be detected by configure.");
 #endif
 	  }
+	  else if (M_lbase == executable_l_addr)
+	    M_lbase = 0;
 
 	  void const* s_end_start_addr;
 	  
 	  if (s_end_offset)
-	  {
 	    s_end_start_addr = (char*)M_lbase + s_end_offset;
-	    M_size = s_end_offset;
-          }
 	  else
-	  {
 	    s_end_start_addr = NULL;
-	    M_size = 0;			// We will determine this later.
-	  }
 
 	  // Sort the symbol table in order of start address.
 	  std::sort(M_symbol_table, &M_symbol_table[M_number_of_symbols], symbol_less());
@@ -572,8 +567,8 @@ void bfd_close(bfd* abfd)
 	        (char*)symbol_start_addr(M_symbol_table[i + 1]) - (char*)symbol_start_addr(M_symbol_table[i]);
 
 	  // Use reasonable size for last one.
-	  // This should be "_end", or one behond it, and will be thrown away in the next loop.
-	  symbol_size(M_symbol_table[M_number_of_symbols - 1]) = 100000;
+	  // This should be "_end", or one beyond it, and will be thrown away in the next loop.
+	  symbol_size(M_symbol_table[M_number_of_symbols - 1]) = 100001;
 
           BFD_ACQUIRE_WRITE_LOCK;	// Needed for M_function_symbols.
 
@@ -600,7 +595,7 @@ void bfd_close(bfd* abfd)
 	    if (((*s)->name[0] == '.' && (*s)->name[1] == 'L')
 	        || ((*s)->flags & (BSF_SECTION_SYM|BSF_OLD_COMMON|BSF_NOT_AT_END|BSF_INDIRECT|BSF_DYNAMIC)) != 0
 		|| (s_end_start_addr != NULL && symbol_start_addr(*s) >= s_end_start_addr)
-		|| (*s)->flags & BSF_FUNCTION == 0)	// Only keep functions.
+		|| ((*s)->flags & BSF_FUNCTION) == 0)	// Only keep functions.
 	    {
 	      *s = *se2--;
 	      --M_number_of_symbols;
@@ -617,31 +612,43 @@ void bfd_close(bfd* abfd)
 	    // M_function_symbols is a set<> that sorts in reverse order (using symbol_key_greater)!
 	    // So use begin() here in order to get the last symbol.
 	    asymbol const* last_symbol = (*M_function_symbols.begin()).get_symbol();
-	    if (symbol_size(last_symbol) == 100000)
+	    if (symbol_size(last_symbol) == 100001)
 	    {
 	      BFD_RELEASE_WRITE_LOCK;
 #if CWDEBUG_ALLOC
-	      int saved_internal = __libcwd_tsd.internal;
 	      __libcwd_tsd.internal = 0;
 #endif
 	      Dout(dc::warning, "Unknown size of symbol " << last_symbol->name);
 #if CWDEBUG_ALLOC
-	      __libcwd_tsd.internal = saved_internal;
+	      __libcwd_tsd.internal = 1;
 #endif
 	      BFD_ACQUIRE_WRITE_LOCK;
 	    }
-	    if (!M_size)
-	      M_size = (char*)symbol_start_addr(last_symbol) + symbol_size(last_symbol) - (char*)M_lbase;
-	    if (M_lbase == 0)
-	    {
-	      asymbol const* first_symbol = (*M_function_symbols.rbegin()).get_symbol();
-	      M_start = symbol_start_addr(first_symbol); // For the executable, use the lowest start address of all symbols.
-	    }
+	    asymbol const* first_symbol = (*M_function_symbols.rbegin()).get_symbol();
+	    M_start = symbol_start_addr(first_symbol); // Use the lowest start address of all symbols.
+	    if (s_end_start_addr)
+	      M_size = (char*)s_end_start_addr - (char*)M_start;
 	    else
-	      M_start = M_lbase;		// Ok for shared libraries.
+	    {
+	      if (symbol_size(last_symbol) == 100001)			// In fact unknown?
+	      {
+		M_start_last_symbol = symbol_start_addr(last_symbol);	// Initialize M_start_last_symbol.
+#if !CWDEBUG_LIBBFD
+		M_size = (char*)last_symbol->section->vma + last_symbol->section->M_size - (char*)M_start;
+		symbol_size(const_cast<asymbol*>(last_symbol)) = M_size - ((char*)symbol_start_addr(last_symbol) - (char*)M_start);
+#endif
+	      }
+#if !CWDEBUG_LIBBFD
+	      else
+#endif
+		M_size = (char*)symbol_start_addr(last_symbol) + symbol_size(last_symbol) - (char*)M_start;
+	    }
 	  }
 	  else
-	    M_start = M_lbase;			// Probably a shared library, need to initialize this anyway.
+	  {
+	    M_start = M_lbase;			// Initialize to arbitrary value.
+	    M_size = 0;
+	  }
 
 #if 0
 	  if (M_function_symbols.size() < 20000)
@@ -649,7 +656,6 @@ void bfd_close(bfd* abfd)
 	    libcwd::debug_ct::OnOffState state;
 	    Debug( libcw_do.force_on(state) );
 #if CWDEBUG_ALLOC
-	    int saved_internal = __libcwd_tsd.internal;
 	    __libcwd_tsd.internal = 0;
 #endif
 	    for(function_symbols_ct::iterator i(M_function_symbols.begin()); i != M_function_symbols.end(); ++i)
@@ -658,7 +664,7 @@ void bfd_close(bfd* abfd)
 	      Dout(dc::always, s->name << " (" << s->section->name << ") = " << s->value);
 	    }
 #if CWDEBUG_ALLOC
-	    __libcwd_tsd.internal = saved_internal;
+	    __libcwd_tsd.internal = 1;
 #endif
 	    Debug( libcw_do.restore(state) );
 	  }
@@ -673,10 +679,82 @@ void bfd_close(bfd* abfd)
 #endif
 	}
 
+	bool already_exists = false;
 	if (M_number_of_symbols > 0)
-	  NEEDS_WRITE_LOCK_object_files().push_back(this);
-
+	{
+#if CWDEBUG_ALLOC
+	  __libcwd_tsd.internal = 0;
+#endif
+	  Dout(dc::bfd, "Adding \"" << get_object_file()->filepath() << "\", load address " <<
+	      get_lbase() << ", start " << get_start() << " and end " << (void*)((size_t)get_start() + size()));
+	  // Find out if there is any overlap.
+	  for (object_files_ct::iterator iter = NEEDS_WRITE_LOCK_object_files().begin();
+	       iter != NEEDS_WRITE_LOCK_object_files().end(); ++iter)
+          {
+	    bool shared_libraries_overlap = (size_t)(*iter)->get_start() < (size_t)get_start() + size() &&
+	        (size_t)(*iter)->get_start() + (*iter)->size() > (size_t)get_start();
+	    if (shared_libraries_overlap)
+	    {
+              if ((*iter)->get_start() == get_start())
+              {
+                LIBCWD_ASSERT((*iter)->size() == size());
+                Dout(dc::bfd, "Already loaded as \"" << (*iter)->get_object_file()->filepath() << "\"");
+                already_exists = true;
+                break;
+              }
+	      Dout(dc::bfd, "It collides with \"" << (*iter)->get_object_file()->filepath() << "\", load address " <<
+		  (*iter)->get_lbase() << ", start " << (*iter)->get_start() << " and end " <<
+		  (void*)((size_t)(*iter)->get_start() + (*iter)->size()));
+	       
+	      bfile_ct* lowbf;
+	      bfile_ct* highbf;
+	      // Real size:
+	      // <----lowbf----><----highbf---->
+	      // Since only the end can be too large, overlap means we have either:
+	      // <----lowbf----------->
+	      //                <----highbf---->
+	      // or
+	      // <----lowbf------------------------>
+	      //                <----highbf---->
+	      // Hence, it suffices to check the start.
+	      if ((size_t)(*iter)->get_start() < (size_t)get_start())
+	      {
+	        lowbf = *iter;
+	        highbf = this;
+	      }
+	      else
+	      {
+	        lowbf = this;
+	        highbf = *iter;
+	      }
+	      LIBCWD_ASSERT(lowbf->M_start_last_symbol);			// Must be set to be variable.
+	      LIBCWD_ASSERT(lowbf->M_start_last_symbol < highbf->get_start());	// At most last symbol size can be wrong.
+	      lowbf->M_size = (char*)highbf->M_start - (char*)lowbf->M_start;
+	      Dout(dc::bfd, "End of \"" << lowbf->get_object_file()->filepath() << "\" corrected to be " <<
+	          (void*)((size_t)lowbf->get_start() + lowbf->size()));
+	      break;
+	    }
+	  }
+#if CWDEBUG_ALLOC
+	  __libcwd_tsd.internal = 1;
+#endif
+          if (!already_exists)
+	    NEEDS_WRITE_LOCK_object_files().push_back(this);
+        }
 	BFD_RELEASE_WRITE_LOCK;
+        if (already_exists)
+	{
+	  free(M_symbol_table);
+	  M_symbol_table  = NULL;
+	  if (M_abfd)
+	  {
+	    bfd_close(M_abfd);
+	    M_abfd = NULL;
+	  }
+	  M_number_of_symbols = 0;
+	  return true;
+	}
+        return false;
       }
 
       void bfile_ct::deinitialize(LIBCWD_TSD_PARAM)
@@ -713,7 +791,7 @@ void bfd_close(bfd* abfd)
 	}
 	set_alloc_checking_on(LIBCWD_TSD);
       }
-#endif // HIDE_FROM_DOXYGEN
+      //! \endcond HIDE_FROM_DOXYGEN
 
       // cwbfd::
       struct object_file_greater {
@@ -1043,7 +1121,7 @@ void bfd_close(bfd* abfd)
 #endif
         if (l_addr == unknown_l_addr)
 	  Dout(dc::bfd|continued_cf|flush_cf, "Loading debug symbols from " << name << ' ');
-	else if (l_addr == 0)
+	else if (l_addr == executable_l_addr)
 	  Dout(dc::bfd|continued_cf|flush_cf, "Loading debug symbols from " << name << "... ");
 	else
 	  Dout(dc::bfd|continued_cf|flush_cf, "Loading debug symbols from " << name << " (" << l_addr << ") ... ");
@@ -1058,19 +1136,21 @@ void bfd_close(bfd* abfd)
 #if CWDEBUG_ALLOC
 	bool is_libc = (strncmp("libc.so", slash + 1, 7) == 0);
 #endif
+        bool already_exists;
 	LIBCWD_DEFER_CANCEL;
 	BFD_ACQUIRE_WRITE_LOCK;
 	set_alloc_checking_off(LIBCWD_TSD);
 	object_file = new bfile_ct(name, l_addr);
 	BFD_RELEASE_WRITE_LOCK;
+	already_exists =
 #if LIBCWD_THREAD_SAFE && CWDEBUG_ALLOC && __GNUC__ == 3 && __GNUC_MINOR__ == 4
-	object_file->initialize(name, l_addr LIBCWD_COMMA_ALLOC_OPT(is_libc), is_libstdcpp LIBCWD_COMMA_TSD);
+	    object_file->initialize(name LIBCWD_COMMA_ALLOC_OPT(is_libc), is_libstdcpp LIBCWD_COMMA_TSD);
 #else
-	object_file->initialize(name, l_addr LIBCWD_COMMA_ALLOC_OPT(is_libc) LIBCWD_COMMA_TSD);
+	    object_file->initialize(name LIBCWD_COMMA_ALLOC_OPT(is_libc) LIBCWD_COMMA_TSD);
 #endif
 	set_alloc_checking_on(LIBCWD_TSD);
 	LIBCWD_RESTORE_CANCEL;
-	if (object_file->get_number_of_symbols() > 0)
+	if (!already_exists && object_file->get_number_of_symbols() > 0)
 	{
 	  Dout(dc::finish, "done (" << std::dec << object_file->get_number_of_symbols() << " symbols)");
 #ifdef LIBCWD_DEBUGBFD
@@ -1079,8 +1159,13 @@ void bfd_close(bfd* abfd)
 	}
 	else
 	{
-	  Dout(dc::finish, "No symbols found");
-	  object_file->deinitialize(LIBCWD_TSD);
+	  if (already_exists)
+	    Dout(dc::finish, "Already loaded");
+	  else
+	  {
+	    Dout(dc::finish, "No symbols found");
+	    object_file->deinitialize(LIBCWD_TSD);
+	  }
 	  set_alloc_checking_off(LIBCWD_TSD);
 	  delete object_file;
 	  set_alloc_checking_on(LIBCWD_TSD);
@@ -1188,7 +1273,7 @@ void bfd_close(bfd* abfd)
 
 	// Load executable
 	BFD_INITIALIZE_LOCK;
-	load_object_file(fullpath->data(), 0);
+	load_object_file(fullpath->data(), executable_l_addr);
 
 	if (!statically_linked)
 	{
@@ -1214,13 +1299,8 @@ void bfd_close(bfd* abfd)
 	  }
 #endif
 #endif
-	    if (l->l_name && (l->l_name[0] == '/') || (l->l_name[0] == '.'))
-	    {
-	      if (l->l_addr)
-		load_object_file(l->l_name, reinterpret_cast<void*>(l->l_addr));
-	      else
-		load_object_file(l->l_name, unknown_l_addr);
-	    }
+	    if (l->l_name && ((l->l_name[0] == '/') || (l->l_name[0] == '.')))
+	      load_object_file(l->l_name, reinterpret_cast<void*>(l->l_addr));
 	  }
 
 	  LIBCWD_DEFER_CANCEL;
@@ -1256,7 +1336,7 @@ void bfd_close(bfd* abfd)
 
 	  // Make symbol_start_addr(&dummy_symbol) and symbol_size(&dummy_symbol) return the correct value:
 	  bfd_asymbol_bfd(&dummy_symbol) = object_file->get_bfd();
-	  dummy_section.offset = 0;			// Use an offset of 0 and
+	  dummy_section.vma = 0;			// Use a vma of 0 and
 	  dummy_symbol.section = &dummy_section;	// use dummy_symbol.value to store (value + offset):
 	  dummy_symbol.value = reinterpret_cast<char const*>(addr) - reinterpret_cast<char const*>(object_file->get_lbase());
 	  symbol_size(&dummy_symbol) = 1;
@@ -1429,24 +1509,30 @@ typedef location_ct bfd_location_ct;
       if (!object_file && !statically_linked)
       {
         // Try to load everything again... previous loaded libraries are skipped based on load address.
-	for(link_map* l = *dl_loaded_ptr; l;)
+	for(link_map* l = *dl_loaded_ptr; l; l = l->l_next)
 	{
-	  if (l->l_addr)
-	  {
-	    BFD_ACQUIRE_READ_LOCK;
-	    for (object_files_ct::const_iterator iter = NEEDS_READ_LOCK_object_files().begin();
-		 iter != NEEDS_READ_LOCK_object_files().end();
-		 ++iter)
-	      if (reinterpret_cast<void*>(l->l_addr) == (*iter)->get_lbase())
+	  bool already_loaded = false;
+	  BFD_ACQUIRE_READ_LOCK;
+	  for (object_files_ct::const_iterator iter = NEEDS_READ_LOCK_object_files().begin();
+	       iter != NEEDS_READ_LOCK_object_files().end(); ++iter)
+          {
+	    if (reinterpret_cast<void*>(l->l_addr) == (*iter)->get_lbase())
+	    {
+	      // This paths are very likely the same, but I don't want to take any risk.
+	      struct stat statbuf1;
+	      struct stat statbuf2;
+	      int res1 = stat(l->l_name, &statbuf1);
+	      int res2 = stat((*iter)->get_object_file()->filepath(), &statbuf2);
+	      if (res1 == 0 && res2 == 0 && statbuf1.st_ino == statbuf2.st_ino)
 	      {
-		BFD_RELEASE_READ_LOCK;
-		goto already_loaded;
+		already_loaded = true;
+		break;
 	      }
-            BFD_RELEASE_READ_LOCK;
-	    load_object_file(l->l_name, reinterpret_cast<void*>(l->l_addr));
+	    }
 	  }
-already_loaded:
-	  l = l->l_next;
+	  BFD_RELEASE_READ_LOCK;
+	  if (!already_loaded && l->l_name && ((l->l_name[0] == '/') || (l->l_name[0] == '.')))
+	    load_object_file(l->l_name, reinterpret_cast<void*>(l->l_addr));
 	}
         BFD_ACQUIRE_WRITE_LOCK;
 	set_alloc_checking_off(LIBCWD_TSD);
