@@ -1,4 +1,4 @@
-// $Header: /cvsroot/libcwd/libcwd/debug.cc,v 1.144 2005/06/29 04:10:24 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/debug.cc,v 1.145 2006/03/07 02:51:36 libcw Exp $
 //
 // Copyright (C) 2000 - 2004, by
 // 
@@ -1883,7 +1883,7 @@ void allocator_unlock(void)
               )
 	  {
 #if CWDEBUG_ALLOC
-	    int saved_internal;
+	    int saved_internal = 0;	// To avoid 'may be used uninitialized' compiler warning.
 	    bool is_internal = __libcwd_tsd.internal;
 	    if (is_internal)
 	      saved_internal = _private_::set_library_call_on(LIBCWD_TSD);	// flush is a library call.
