@@ -1,4 +1,4 @@
-// $Header: /cvsroot/libcwd/libcwd/include/cwd_bfd.h,v 1.21 2005/06/25 02:36:44 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/include/cwd_bfd.h,v 1.25 2005/12/13 01:39:18 libcw Exp $
 //
 // Copyright (C) 2002 - 2004, by
 //
@@ -99,7 +99,7 @@ public:
 };
 
 struct symbol_key_greater {
-  // Returns true when the start of a lays behond the end of b (ie, no overlap).
+  // Returns true when the start of a lays beyond the end of b (ie, no overlap).
   bool operator()(symbol_ct const& a, symbol_ct const& b) const;
 };
 
@@ -121,9 +121,10 @@ typedef std::list<bfile_ct*> object_files_ct;
 class bfile_ct {                                  // All allocations related to bfile_ct must be `internal'.
 private:
   bfd* M_abfd;
-  void* M_lbase;	// The 'load address', or 0 for the executable.
-  void const* M_start;	// For shared libraries, this is equal to M_lbase.
-  size_t M_size;
+  void* M_lbase;			// The 'load address', or 0 for the executable.
+  void const* M_start;			// The start address of the first symbol.
+  void const* M_start_last_symbol;	// The start address of the last symbol, or 0 if not relevant.
+  size_t M_size;			// The difference between M_start and the end of the last symbol.
   asymbol** M_symbol_table;
   long M_number_of_symbols;
   function_symbols_ct M_function_symbols;
@@ -131,9 +132,9 @@ private:
 public:
   bfile_ct(char const* filename, void* base);
 #if LIBCWD_THREAD_SAFE && CWDEBUG_ALLOC && __GNUC__ == 3 && __GNUC_MINOR__ == 4
-  void initialize(char const* filename, void* base LIBCWD_COMMA_ALLOC_OPT(bool is_libc), bool is_libstdcpp LIBCWD_COMMA_TSD_PARAM);
+  bool initialize(char const* filename LIBCWD_COMMA_ALLOC_OPT(bool is_libc), bool is_libstdcpp LIBCWD_COMMA_TSD_PARAM);
 #else
-  void initialize(char const* filename, void* base LIBCWD_COMMA_ALLOC_OPT(bool is_libc) LIBCWD_COMMA_TSD_PARAM);
+  bool initialize(char const* filename LIBCWD_COMMA_ALLOC_OPT(bool is_libc) LIBCWD_COMMA_TSD_PARAM);
 #endif
   void deinitialize(LIBCWD_TSD_PARAM);
 
@@ -189,7 +190,7 @@ inline bfd* bfd_asymbol_bfd(asymbol const* s) { return s->bfd_ptr; }
 inline addr_const_ptr_t
 symbol_start_addr(asymbol const* s)
 {
-  return s->value + bfd_get_section(s)->offset
+  return s->value + bfd_get_section(s)->vma
       + reinterpret_cast<char const*>(reinterpret_cast<bfile_ct const*>(bfd_asymbol_bfd(s)->usrdata)->get_lbase());
 }
 
