@@ -1,4 +1,4 @@
-// $Header: /cvsroot/libcwd/libcwd/include/elf32.h,v 1.12 2004/06/03 12:23:07 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/include/elf32.h,v 1.14 2005/12/13 21:00:27 libcw Exp $
 //
 // Copyright (C) 2001 - 2004, by
 //
@@ -67,8 +67,8 @@ struct asymbol_st {
 
 struct asection_st {
   Elf32_Addr vma;
-  Elf32_Addr offset;
   char const* name;
+  Elf32_Word M_size;	// This breaks libbfd compatibility I guess.
 };
 
 struct bfd_st {
@@ -84,7 +84,7 @@ public:
   bfd_st(void) : M_has_syms(false) { }
   virtual ~bfd_st() { }
 public:
-  static bfd_st* openr(char const* file_name, bool shared_library);
+  static bfd_st* openr(char const* file_name);
   virtual void close(void) = 0;
   virtual bool check_format(void) const = 0;
   virtual long get_symtab_upper_bound(void) = 0;
