@@ -1,6 +1,6 @@
-// $Header: /cvsroot/libcwd/libcwd/include/ios_base_Init.h,v 1.8 2004/05/27 03:03:50 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/include/ios_base_Init.h,v 1.11 2004/10/07 14:09:09 libcw Exp $
 //
-// Copyright (C) 2000 - 2003, by
+// Copyright (C) 2000 - 2004, by
 // 
 // Carlo Wood, Run on IRC <carlo@alinoe.com>
 // RSA-1024 0x624ACAD5 1997-01-26                    Sign & Encrypt
@@ -19,7 +19,10 @@
 namespace libcwd {
   namespace _private_ {
 
-#if __GNUC_MINOR__ < 3
+// Initialization 3.3 and later is allocation free (emperically) if this condition is met:
+#define LIBCWD_IOSBASE_INIT_ALLOCATES 1
+
+#if LIBCWD_IOSBASE_INIT_ALLOCATES
 extern bool WST_ios_base_initialized;
 extern bool inside_ios_base_Init_Init(void);
 #endif
