@@ -1,4 +1,4 @@
-// $Header: /cvsroot/libcwd/libcwd/testsuite/libcwd.tst/sys.h,v 1.10 2003/04/17 00:38:44 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/testsuite/libcwd.tst/sys.h,v 1.11 2004/09/01 23:40:55 libcw Exp $
 //
 // Copyright (C) 2000 - 2003, by
 //
@@ -14,7 +14,7 @@
 #ifndef TESTSUITE_SYS_H
 #define TESTSUITE_SYS_H
 
-#include "../../config.h"
+#include "config.h"
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
