@@ -1,6 +1,6 @@
-// $Header: /cvsroot/libcwd/libcwd/utils/attach_gdb.cc,v 1.2 2004/05/27 03:03:56 libcw Exp $
+// $Header: /cvsroot/libcwd/libcwd/utils/attach_gdb.cc,v 1.6 2004/09/01 23:40:55 libcw Exp $
 //
-// Copyright (C) 2003, by
+// Copyright (C) 2003 - 2004, by
 //
 // Carlo Wood, Run on IRC <carlo@alinoe.com>
 // RSA-1024 0x624ACAD5 1997-01-26                    Sign & Encrypt
@@ -13,8 +13,24 @@
 
 #include "sys.h"
 #include "cwd_debug.h"
+
+#ifdef _WIN32
+
+namespace libcwd {
+
+void attach_gdb(void)
+{
+  DoutFatal(dc::core, "attach_gdb() is not supported on windows");
+}
+
+}
+
+#else
+
 #include <unistd.h>
+#include <sys/types.h>
 #include <sys/wait.h>
+#include <cerrno>
 #include <time.h>
 #include <fstream>
 
@@ -56,10 +72,32 @@ void attach_gdb(void)
     {
       struct timespec t = { 0, 100000000 };
       while (libcwd_attach_gdb_hook)
+      {
+        int status;
+        pid_t pid3 = waitpid(pid2, &status, WNOHANG);
+        if (pid3 == pid2 || ((int)pid3 == -1 && errno == ECHILD))
+	{
+          libcwd_attach_gdb_hook = 0;
+	  if (WIFEXITED(status))
+	    DoutFatal(dc::core, "Failed to start gdb: 'xterm' terminated with exit code " << WEXITSTATUS(status) <<
+	        " before attaching to the process.  This can for instance happen when you call attach_gdb from "
+		"the destructor of a global object.");
+	  else if (WIFSIGNALED(status))
+	    DoutFatal(dc::core, "Failed to start gdb: 'xterm' terminated because of (uncaught) signal " << WTERMSIG(status) <<
+	        " before attaching to the process.");
+#ifdef WCOREDUMP
+          else if (WCOREDUMP(status))
+	    DoutFatal(dc::core, "Failed to start gdb: 'xterm' dumped core before attaching to the process.");
+#endif
+	  DoutFatal(dc::core, "Failed to start gdb: 'xterm' terminated before attaching to the process.");
+	}
 	nanosleep(&t, NULL);
+      }
       Dout(dc::always, "ATTACHED!");
     }
   }
 }
 
 } // namespace libcwd
+
+#endif // !_WIN32
